@@ -16,7 +16,7 @@ try:
 except ImportError:
     import urllib.request as urllib2
 
-__version__ = "0.1.22"
+__version__ = "0.1.23"
 __author__ = "Xiaocong He"
 __all__ = ["device", "Device", "rect", "point", "Selector"]
 
@@ -93,13 +93,20 @@ class JsonRPCMethod(object):
                 raise Exception("Error reponse from jsonrpc server.")
             jsonresult = json.loads(res.data.decode("utf-8"))
         else:
-            req = urllib2.Request(self.url,
-                                  json.dumps(data).encode("utf-8"),
-                                  {"Content-type": "application/json"})
-            result = urllib2.urlopen(req, timeout=self.timeout)
-            if result is None or result.getcode() != 200:
-                raise Exception("Error reponse from jsonrpc server.")
-            jsonresult = json.loads(result.read().decode("utf-8"))
+            result = None
+            try:
+                req = urllib2.Request(self.url,
+                                      json.dumps(data).encode("utf-8"),
+                                      {"Content-type": "application/json"})
+                result = urllib2.urlopen(req, timeout=self.timeout)
+                if result is None or result.getcode() != 200:
+                    raise Exception("Error reponse from jsonrpc server.")
+                jsonresult = json.loads(result.read().decode("utf-8"))
+            except:
+                raise
+            finally:
+                if result is not None:
+                    result.close()
         if "error" in jsonresult and jsonresult["error"]:
             raise Exception("Error response. Error code: %d, Error message: %s" %
                             (jsonresult["error"]["code"], jsonresult["error"]["message"]))
@@ -336,7 +343,15 @@ class AutomatorServer(object):
 
     def download(self, filename, url):
         with open(filename, 'wb') as file:
-            file.write(urllib2.urlopen(url).read())
+            res = None
+            try:
+                res = urllib2.urlopen(url)
+                file.write(res.read())
+            except:
+                raise
+            finally:
+                if res is not None:
+                    res.close()
 
     @property
     def jsonrpc(self):
@@ -346,7 +361,7 @@ class AutomatorServer(object):
         return self.__jsonrpc()
 
     def __jsonrpc(self):
-        return JsonRPCClient(self.rpc_uri)
+        return JsonRPCClient(self.rpc_uri, timeout=int(os.environ.get("JSONRPC_TIMEOUT", 90)))
 
     def start(self):
         files = self.download_and_push()
@@ -381,12 +396,15 @@ class AutomatorServer(object):
     def stop(self):
         '''Stop the rpc server.'''
         if self.uiautomator_process and self.uiautomator_process.poll() is None:
+            res = None
             try:
-                urllib2.urlopen(self.stop_uri)
+                res = urllib2.urlopen(self.stop_uri)
                 self.uiautomator_process.wait()
             except:
                 self.uiautomator_process.kill()
             finally:
+                if res is not None:
+                    res.close()
                 self.uiautomator_process = None
         try:
             out = self.adb.cmd("shell", "ps", "-C", "uiautomator").communicate()[0].decode("utf-8").strip().splitlines()
@@ -941,7 +959,7 @@ class AutomatorDeviceObject(AutomatorDeviceUiObject):
     def up(self, **kwargs):
         def above(rect1, rect2):
             left, top, right, bottom = intersect(rect1, rect2)
-            return rect1["top"] - rect2["bottom"] if left < right else -1 
+            return rect1["top"] - rect2["bottom"] if left < right else -1
         return self.__view_beside(above, **kwargs)
 
     def down(self, **kwargs):
